<?php
// HTTP Agent 0.1.0
// (C)Copyright 2001, 2002 Jon D. Frisby
// jfrisby@mrjoy.com
// http://www.mrjoy.com
//
// TODO:
// -HTTP 1.1:
//    -Keepalive.
//    -Transfer encoding (types other than chunked?).
// -Customizable connect/transfer timeouts.
// -----ajout 
// -HTTP authentication.
// -Cookies (cookie jar, cookie parsing, etc...).
//-----------
// -Binary safety.
// -What are "correct" semantics for a redirect response from a POST?
// 
// -------------------------------------------------------------------
// Important Note:
// Be sure to call set_time_limit(...) with a suitably large value!
// -------------------------------------------------------------------
// 
// 
// -------------------------------------------------------------------
// http_agent - A class that allows one to make HTTP GET and POST 
// requests.
// -------------------------------------------------------------------
//
// Classe corrige par Michel Dubois

class HTTP_Agent {
    // User-controllable state.
    var $protocol; // Must be one of "http", or "https".
    var $host;
    var $port;

    // Internal state.
    var $fd = null;

    var $errcode = null;

    // User-controllable configuration.
    var $binarysafe = false;
    var $http_version = "1.1";
    var $follow_redirects = true;

    // Stuff that should be user-controllable.
    var $connect_timeout = 30;

    var $isopen = false;
    
    var $baseref="";
    
    var $cookie="";
    
    var $authorization="";
    
    
    function set_baseref($baseref) {
        $this->baseref=$baseref;
    }
    
    function get_baseref() {return $this->beseref;}
    
    function set_cookie($cookie) {
        $this->cookie=$cookie;
    }
    
    function get_cookie() {return $this->cookie;}  
        
    function set_authorization($authorization) {
        $this->authorization=$authorization;
    }
    
    function get_authorization() {return $this->authorization;}
    
    function http_agent($protocol_p, $host_p, $port_p) {
        $this->set_protocol($protocol_p);
        $this->set_host($host_p);
        $this->set_port($port_p);
        $this->baseref="";
        $this->cookie="";
    }

    // Sets the protocol to use.  Must be one of HTTP or HTTPS.  Upon
    // successfully *changing* the protocol, any open connection will
    // be closed.  If the protocol is set to the same value as the 
    // current protocol, nothing is changed.  
    //
    // Returns the most recent error code, or null if everything's OK.

    function set_protocol($protocol_p) {
        $protocol_p = strtolower($protocol_p);
        if($protocol_p == "http" or $protocol_p == "https") {
            if($this->protocol != $protocol_p) {
                $this->protocol = $protocol_p;
                $this->close();
            }
        } else {
            $this->errcode = HTTP_ERR_INVALID_PROTOCOL();
        }

        return $this->errcode;
    }
    function get_protocol() { return $this->protocol; }

    // Sets what version of HTTP to use.  Any open connection will be 
    // closed, unless the new value is identical to the current one.  
    // An attempt to actually connect is not immediately made.
    //
    // Returns the most recent error code, or null if everything's OK.
    function set_http_version($ver_p) {
        if($ver_p != '1.0' and $ver_p != '1.1') {
            $this->errcode = HTTP_ERR_INVALID_HTTP_VERSION();
        } else {
            if($ver_p != $this->http_version) {
                $this->http_version = $ver_p;
                $this->close();
            }
        }

        return $this->errcode;
    }
    function get_http_version() { return $this->http_version; }

    // Sets whether or not to follow redirects.  
    //
    // Returns the most recent error code, or null if everything's OK.
    function set_follow_redirects($follow_redirects_p) {
        if($follow_redirects_p) { $this->follow_redirects = true; } 
        else { $this->follow_redirects = false; }

        return $this->errcode;
    }
    function get_follow_redirects() { return $this->follow_redirects; }

    // Sets the host to connect to.  Any open connection will be 
    // closed, unless the new host value is identical to the current 
    // one.  An attempt to actually connect is not immediately made.
    //
    // Returns the most recent error code, or null if everything's OK.
    function set_host($host_p) {
        $host_p = strtolower($host_p);
        if($this->host != $host_p) {
            $this->host = $host_p;
            $this->close();
        }

        return $this->errcode;
    }
    function get_host() { return $this->host; }

    // Sets the port to connect on.  Any open connection will be 
    // closed, unless the new port value is identical to the current 
    // one.  If a null value is given, then the "normal" port for the
    // protocol that is in use at the time of the next connection 
    // attempt will be used.
    //
    // Returns the most recent error code, or null if everything's OK.
    function set_port($port_p) {
        $port_p = strtolower($port_p);
        if($this->port != $port_p) {
            $this->port = $port_p;
            $this->close();
        }

        return $this->errcode;
    }
    function get_port() { return $this->port; }

    // Sets whether or not to ensure that binary-safe file I/O is 
    // performed during requests.
    //
    // Returns the most recent error code, or null if everything's OK.
    function set_binarysafe($binarysafe_p) {
        if($binarysafe_p) { $this->binarysafe = true; }
        else { $this->binarysafe = false; }
    }
    function get_binarysafe() { return $this->binarysafe; }

    // Reset the error state to null.
    function clear_error() { $this->errcode = null; }

    // Retrieve the most recently encountered error.
    function get_errcode() { return $this->errcode; }

    // Perform a GET request.  The specified path should be devoid of
    // protocol/host specifiers, but should be an absolute path.  If
    // parameters are already attached to it, then any parameters 
    // specified in addition will be added.
    //
    // Request parameters are passed as an associate array in the 
    // second parameter.
    function get($path, $params) {
        $parts = split('\?', $path);
        $suffix = stringify_params($params);
        if(count($parts)>1 and !is_null($parts[1]) and $parts[1] != '') { $suffix = $parts[1] . "&" . $suffix; }
        if($suffix != "") { $suffix = $parts[0] . "?" . $suffix; }
        else { $suffix = $parts[0]; }

        $request = formulate_request($this, $suffix, "GET", null,$this->cookie, $this->authorization);
        $request .= "\r\n";
        print $request;
        
        $this->open();
        $r = null;
        if(!is_null($this->fd)) {
            fwrite($this->fd, $request, strlen($request));
            $r = read_response($this);
            $h=$r->get_headers();
            if (array_key_exists('Set-Cookie',$h)) {
                      $p = split(";", $h['Set-Cookie']);
                      $this->cookie=$p[0];
            }
            $this->close();

            if($this->follow_redirects and $r->get_code() >= 300 and $r->get_code() <= 399) {
                $l = $r->get_header('Location');
                if(!is_null($l) and $l != '') { $r = $this->get($this->baseref.$l, null); }
            }
        }

        return $r;
    }

    // Perform a POST request.  The specified path should be devoid of
    // protocol/host specifiers, but should be an absolute path.  No 
    // parameters should be directly specified in the path.
    //
    // Request parameters are passed as an associate array in the 
    // second parameter.
    function post($path, $params) {
        $body = stringify_params($params);

        $request = formulate_request($this, $path, "POST", $body,$this->cookie,$this->authorization);

        $this->open();
        $r = null;
        if(!is_null($this->fd)) {
            fwrite($this->fd, $request, strlen($request));
            $r = read_response($this);
            $h=$r->get_headers();
            if (array_key_exists('Set-Cookie',$h)) {
                      $p = split(";", $h['Set-Cookie']);
                      $this->cookie=$p[0];
            }
            $this->close();

            if($this->follow_redirects and $r->get_code() >= 300 and $r->get_code() <= 399) {
                $l = $r->get_header('Location');
                if(!is_null($l) and $l != '') {
                    $r = $this->post($this->baseref.$l, null);
                    // TODO: How should POST redirects work???
                }
            }
        }

        return $r;
    }

    // ---------------------------------------------------------------
    // Private Methods - Do not call these directly.
    // ---------------------------------------------------------------

    // If a connection is open, close it.
    function close() {
        if($this->isopen) { 
            $this->isopen = false;
            if(!fclose($this->fd)) { $this->errcode = HTTP_ERR_FCLOSE(); }
        }
    }

    // Open a connection.
    function open() {
        global $portmap;
        if(!$this->isopen) {
            $hosttmp = $this->host;
            if($this->protocol == 'https') { $hosttmp = 'ssl://' . $hosttmp; }

            $porttmp = $this->port;
            if(is_null($porttmp)) { $porttmp = $portmap[$this->protocol]; }

            $this->fd = fsockopen($hosttmp, $porttmp, $errno, $errstr, $this->connect_timeout);
            if($errno or !($this->fd)) {
                $this->errcode = HTTP_ERR_FSOCKOPEN();
                $this->fd = null;
            }
            else
              $this->isopen = true;
        }
    }
}



// -------------------------------------------------------------------
// http_response - Encapsulate an HTTP response message for convenient
// access.  Headers can be accessed as an associative array, or looked
// up individually by name for the sake of convenience.
//
// If a header occurs multiple times, then an array containing each of
// the values is returned.
// -------------------------------------------------------------------
class http_response {
    var $code;
    var $headers;
    var $body;

    function http_response($code_p, $headers_p, $body_p) {
        $this->code = $code_p;
        $this->headers = $headers_p;
        $this->body = $body_p;
    }

    function get_code()        { return $this->code; }
    function get_body()        { return $this->body; }
    function get_headers()     { return $this->headers; }
    function get_header($name) { return $this->headers[$name]; }
}



// -------------------------------------------------------------------
// Support Functions - These should not be called directly.
// -------------------------------------------------------------------
function stringify_params($params) {
    // TODO: Need to handle complex values.
    $suffix = "";
    if(is_array($params)) {
        $suffix_parts = array();
        foreach($params as $key => $val) {
            $val = urlencode($val);
            $suffix_parts[] = "$key=$val";
        }
        $suffix = join("&", $suffix_parts);
    }

    return $suffix;
}

function formulate_request($http_obj, $path, $method, $body,$cookie="",$authorization="") {
    $request_str = $method . " " . $path . " HTTP/" . $http_obj->http_version . "\r\n";
  
    if($http_obj->http_version == "1.1") { 
        $request_str .= "Host: " . $http_obj->get_host() . "\r\n"; 
        // TODO: Only do this if we're not doing persistent connections.
        $request_str .= "Connection: close\r\n";
    }

    //$request_str .= "User-Agent:	Mozilla/5.0 (Windows; U; Windows NT 5.1; fr; rv:1.6) Gecko/20040206 Firefox/0.8\r\n";

    if (isset($authorization) && $authorization!="")
  //      $request_str .= "Accept: */*\r\n";
        $request_str .= "Authorization: Basic ".base64_encode($authorization)."\r\n";

    if (isset($cookie) && $cookie!="")
        $request_str .= "Cookie: ".$cookie."\r\n";    
    
    if($method == 'POST') {
        $request_str .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $request_str .= "Content-Length: " . strlen($body) . "\r\n";
        $request_str .= "\r\n";
        $request_str .= $body;
    }
    return $request_str;
}

function read_response($http_obj) {
    $response_code = "";
    $response_header = array();
    $response_body = "";
    $mode = 0;
    while(!feof($http_obj->fd)) {
        $line = fgets($http_obj->fd, 1024);
        if($mode == 0 or $mode == 1) {
            if($line == "\n" or $line == "\r\n") {
                $mode = 2;
            } else {
                if($mode == 0) {
                    // Get response code...
                    $response_code = preg_replace("/^.*?\s+(\d+)\s+.*$/s", "\\1", $line);
                    $mode = 1;
                } else {
                    $parts = split(": ", $line);
                    
                    if(array_key_exists($parts[0],$response_header)) {
                        if(!is_array($response_header[$parts[0]])) {
                            $response_header[$parts[0]] = array($response_header[$parts[0]]);
                        }
                        $response_header[$parts[0]][] = trim($parts[1]);
                    } else {
                        $response_header[$parts[0]] = trim($parts[1]);
                    }
                }
            }
        } else {
            $response_body .= $line;
        }
    }
    if(array_key_exists('Transfer-Encoding',$response_header)) {
      $encoding = strtolower($response_header['Transfer-Encoding']);
      if($encoding == 'chunked') {
        $response_body = chunk_decode($response_body); 
      }
    }
    
    return new http_response($response_code, $response_header, $response_body);
}

function chunk_decode($body) {
    $pos = 0;
    $newline_pos = @strpos($body, "\r\n", $pos);
    $size = hexdec(chop(substr($body, $pos, $newline_pos)));
    $chunk_pos = $newline_pos + 2; // Newline length.

    $buffer = "";
    while($size > 0) {
        $buffer .= substr($body, $chunk_pos, $size);
        $pos = $chunk_pos + $size + 2;

        if(strlen($body) > $pos) {
            $newline_pos = @strpos($body, "\r\n", $pos);
            $size = hexdec(chop(substr($body, $pos, $newline_pos - $pos)));
            $chunk_pos = $newline_pos + 2; // Newline length.
        } else {
            $size = 0;
        }
    }

    return $buffer;
}

global $portmap;
$portmap = array(
    'http' => 80,
    'https' => 443,
);

// TODO: Add binary-safety stuff:
// if(is_set($http_obj->binarysafe)) { $old_magic = get_magic_quotes_runtime(); set_magic_quotes_runtime(0); }
// 
// if(is_set($http_obj->binarysafe)) { set_magic_quotes_runtime($old_magic); }



// -------------------------------------------------------------------
// Error Codes - These methods are constants that represent possible
// error conditions.
// -------------------------------------------------------------------

// Indicates that the specified protocol was not valid.
function HTTP_ERR_INVALID_PROTOCOL() { return 1; }

// An error was encountered while closing the socket.
function HTTP_ERR_FCLOSE() { return 2; }

// An error was encountered while opening the socket.
function HTTP_ERR_FSOCKOPEN() { return 3; }

// Indicates that you attempted to specify an unrecognized version of
// the HTTP protocol.
function HTTP_ERR_INVALID_HTTP_VERSION() { return 4; }

?>
