
<?php
function make_timestamp($year, $month=1, $day=1, $hour=0, $minute=0, $second=0, $timezone=99) {
/// Given date parts in user time, produce a GMT timestamp

    $timezone = get_user_timezone($timezone);

    if (abs($timezone) > 13) {
        return mktime((int)$hour,(int)$minute,(int)$second,(int)$month,(int)$day,(int)$year);
    } else {
        $time = gmmktime((int)$hour,(int)$minute,(int)$second,(int)$month,(int)$day,(int)$year);
        return usertime($time, $timezone);  // This is GMT
    }
}


function usergetdate($date, $timezone=99) {
/// Given a $date timestamp in GMT, returns an array
/// that represents the date in user time
    setlocale (LC_TIME, "fr_FR");
    $timezone = get_user_timezone($timezone);
    if (abs($timezone) > 13) {
        return getdate($date);
    }
    //There is no gmgetdate so I have to fake it...
    $date = $date + (int)($timezone * 3600);
    $getdate["seconds"] = gmstrftime("%S", $date);
    $getdate["minutes"] = gmstrftime("%M", $date);
    $getdate["hours"]   = gmstrftime("%H", $date);
    $getdate["mday"]    = gmstrftime("%d", $date);
    $getdate["wday"]    = gmstrftime("%u", $date);
    $getdate["mon"]     = gmstrftime("%m", $date);
    $getdate["year"]    = gmstrftime("%Y", $date);
    $getdate["yday"]    = gmstrftime("%j", $date);
    $getdate["weekday"] = gmstrftime("%A", $date);
    $getdate["month"]   = gmstrftime("%B", $date);
    return $getdate;
}

function usertime($date, $timezone=99) {
/// Given a GMT timestamp (seconds since epoch), offsets it by
/// the timezone.  eg 3pm in India is 3pm GMT - 7 * 3600 seconds

    $timezone = get_user_timezone($timezone);
    if (abs($timezone) > 13) {
        return $date;
    }
    return $date - (int)($timezone * 3600);
}

function usergetmidnight($date, $timezone=99) {
/// Given a time, return the GMT timestamp of the most recent midnight
/// for the current user.

    $timezone = get_user_timezone($timezone);
    $userdate = usergetdate($date, $timezone);

    if (abs($timezone) > 13) {
        return mktime(0, 0, 0, $userdate["mon"], $userdate["mday"], $userdate["year"]);
    }

    $timemidnight = gmmktime (0, 0, 0, $userdate["mon"], $userdate["mday"], $userdate["year"]);
    return usertime($timemidnight, $timezone); // Time of midnight of this user's day, in GMT

}

function usertimezone($timezone=99) {
/// Returns a string that prints the user's timezone

    $timezone = get_user_timezone($timezone);

    if (abs($timezone) > 13) {
        return "server time";
    }
    if (abs($timezone) < 0.5) {
        return "GMT";
    }
    if ($timezone > 0) {
        return "GMT+$timezone";
    } else {
        return "GMT$timezone";
    }
}

function get_user_timezone($tz = 99) {
// Returns a float which represents the user's timezone difference from GMT in hours
// Checks various settings and picks the most dominant of those which have a value

    // Variables declared explicitly global here so that if we add
    // something later we won't forget to global it...
    $timezones = array(
        isset($GLOBALS['USER']->timezone) ? $GLOBALS['USER']->timezone : 99,
        isset($GLOBALS['CFG']->timezone) ? $GLOBALS['CFG']->timezone : 99,
        );
    while($tz == 99 && $next = each($timezones)) {
        $tz = (float)$next['value'];
    }

    return $tz;
}

function format_time($totalsecs, $str=NULL) {
/// Given an amount of time in seconds, returns string
/// formatted nicely as months, days, hours etc as needed

    $totalsecs = abs($totalsecs);

    if (!$str) {  // Create the str structure the slow way
        $str->day   = "jour";
        $str->days  = "jours";
        $str->hour  = "heure";
        $str->hours = "heures";
        $str->min   = "min";
        $str->mins  = "mins";
        $str->sec   = "sec";
        $str->secs  = "secs";
    }

    $days      = floor($totalsecs/86400);
    $remainder = $totalsecs - ($days*86400);
    $hours     = floor($remainder/3600);
    $remainder = $remainder - ($hours*3600);
    $mins      = floor($remainder/60);
    $secs      = $remainder - ($mins*60);

    $ss = ($secs == 1)  ? $str->sec  : $str->secs;
    $sm = ($mins == 1)  ? $str->min  : $str->mins;
    $sh = ($hours == 1) ? $str->hour : $str->hours;
    $sd = ($days == 1)  ? $str->day  : $str->days;

    $odays = "";
    $ohours = "";
    $omins = "";
    $osecs = "";

    if ($days)  $odays  = "$days $sd";
    if ($hours) $ohours = "$hours $sh";
    if ($mins)  $omins  = "$mins $sm";
    if ($secs)  $osecs  = "$secs $ss";

    if ($days)  return "$odays $ohours";
    if ($hours) return "$ohours $omins";
    if ($mins)  return "$omins $osecs";
    if ($secs)  return "$osecs";
    return "maintenant";
}

function userdate($date, $format="", $timezone=99, $fixday = true) {
/// Returns a formatted string that represents a date in user time
/// WARNING: note that the format is for strftime(), not date().
/// Because of a bug in most Windows time libraries, we can't use
/// the nicer %e, so we have to use %d which has leading zeroes.
/// A lot of the fuss below is just getting rid of these leading
/// zeroes as efficiently as possible.
///
/// If parammeter fixday = true (default), then take off leading
/// zero from %d, else mantain it.

    $toto = setlocale (LC_TIME, "fr_FR");
    if ($format == "") {
//        $format = '%%A %%d %%B %%Y, %%H:%%M';
        $format = '%A %d %B %Y, %H:%M';

    }

    $formatnoday = str_replace("%d", "DD", $format);
    if ($fixday) {
        $fixday = ($formatnoday != $format);
    }

    $timezone = get_user_timezone($timezone);
    if (abs($timezone) > 13) {
        if ($fixday) {
            $datestring = strftime($formatnoday, $date);
            $daystring  = str_replace(" 0", "", strftime(" %d", $date));
            $datestring = str_replace("DD", $daystring, $datestring);
        } else {
            $datestring = strftime($format, $date);
        }
    } else {
        $date = $date + (int)($timezone * 3600);
        if ($fixday) {
            $datestring = gmstrftime($formatnoday, $date);
            $daystring  = str_replace(" 0", "", gmstrftime(" %d", $date));
            $datestring = str_replace("DD", $daystring, $datestring);
        } else {
            $datestring = gmstrftime($format, $date);
        }
    }

    return $datestring;
}
function getGmt($time){
  return $time-date('Z',$time);
}
function getGmtDiff($lt, $ut){
  return ($lt-$ut) / 3600;
}
function getMysqlDate($timestamp){
  return date("Y-m-d", $timestamp);
}
function getMysqlDatetime($timestamp){
  return date("Y-m-d H:i:s", $timestamp);
}
function getTimestamp($mysql_datetime){
  return strtotime($mysql_datetime);
}
function checked(&$var, $set_value = 1, $unset_value = 0) {
/// if variable is set, set it to the set_value otherwise set it to the
/// unset_value.  used to handle checkboxes when you are expecting them from
/// a form

    if (empty($var)) {
        $var = $unset_value;
    } else {
        $var = $set_value;
    }
}

function frmchecked(&$var, $true_value = "checked", $false_value = "") {
/// prints the word "checked" if a variable is true, otherwise prints nothing,
/// used for printing the word "checked" in a checkbox form input

    if ($var) {
        echo $true_value;
    } else {
        echo $false_value;
    }
}


function link_to_popup_window ($url, $name="popup", $linkname="click here",
                               $height=400, $width=500, $title="Popup window", $options="none", $return=false) {
/// This will create a HTML link that will work on both
/// Javascript and non-javascript browsers.
/// Relies on the Javascript function openpopup in javascript.php
/// $url must be relative to home page  eg /mod/survey/stuff.php

    global $CFG;

    if ($options == "none") {
        $options = "menubar=0,location=0,scrollbars,resizable,width=$width,height=$height";
    }
    $fullscreen = 0;

    $link = "<a target=\"$name\" title=\"$title\" href=\"$CFG->wwwroot$url\" ".
           "onClick=\"return openpopup('$url', '$name', '$options', $fullscreen);\">$linkname</a>\n";
    if ($return) {
        return $link;
    } else {
        echo $link;
    }
}


function button_to_popup_window ($url, $name="popup", $linkname="click here",
                                 $height=400, $width=500, $title="Popup window", $options="none") {
/// This will create a HTML link that will work on both
/// Javascript and non-javascript browsers.
/// Relies on the Javascript function openpopup in javascript.php
/// $url must be relative to home page  eg /mod/survey/stuff.php

    global $CFG;

    if ($options == "none") {
        $options = "menubar=0,location=0,scrollbars,resizable,width=$width,height=$height";
    }
    $fullscreen = 0;

    echo "<input type=\"button\" name=\"popupwindow\" title=\"$title\" value=\"$linkname ...\" ".
         "onClick=\"return openpopup('$url', '$name', '$options', $fullscreen);\">\n";
}


function close_window_button() {
/// Prints a simple button to close a window

    echo "<center>\n";
    echo "<script>\n";
    echo "<!--\n";
    echo "document.write('<form>');\n";
    echo "document.write('<input type=\"button\" onClick=\"self.close();\" value=\"".get_string("closewindow")."\" />');\n";
    echo "document.write('</form>');\n";
    echo "-->\n";
    echo "</script>\n";
    echo "<noscript>\n";
    echo "<a href=\"".$_SERVER['HTTP_REFERER']."\"><---</a>\n";
    echo "</noscript>\n";
    echo "</center>\n";
}

function choose_from_menu ($options, $name, $selected="", $nothing="choose", $script="", $nothingvalue="0", $return=false) {
/// Given an array of value, creates a popup menu to be part of a form
/// $options["value"]["label"]

    if ($nothing == "choose") {
        $nothing = get_string("choose")."...";
    }

    if ($script) {
        $javascript = "onChange=\"$script\"";
    } else {
        $javascript = "";
    }

    $output = "<select name=\"$name\" $javascript>\n";
    if ($nothing) {
        $output .= "   <option value=\"$nothingvalue\"\n";
        if ($nothingvalue === $selected) {
            $output .= " selected=\"true\"";
        }
        $output .= ">$nothing</option>\n";
    }
    if (!empty($options)) {
        foreach ($options as $value => $label) {
            $output .= "   <option value=\"$value\"";
            if ($value == $selected) {
                $output .= " selected=\"true\"";
            }
            if ($label === "") {
                $output .= ">$value</option>\n";
            } else {
                $output .= ">$label</option>\n";
            }
        }
    }
    $output .= "</select>\n";

    if ($return) {
        return $output;
    } else {
        echo $output;
    }
}

function print_date_selector($day, $month, $year, $currenttime=0) {
// Currenttime is a default timestamp in GMT
// Prints form items with the names $day, $month and $year

    if (!$currenttime) {
        $currenttime = time();
    }
    $currentdate = usergetdate($currenttime);

    for ($i=1; $i<=31; $i++) {
        $days[$i] = "$i";
    }
    for ($i=1; $i<=12; $i++) {
        $months[$i] = userdate(gmmktime(12,0,0,$i,1,2005), "%B");
    }
    for ($i=2005; $i<=2015; $i++) {
        $years[$i] = $i;
    }
    choose_from_menu($days,   $day,   $currentdate['mday'], "");
    choose_from_menu($months, $month, $currentdate['mon'],  "");
    choose_from_menu($years,  $year,  $currentdate['year'], "");
}

function print_time_selector($hour, $minute, $currenttime=0, $step=5) {
// Currenttime is a default timestamp in GMT
// Prints form items with the names $hour and $minute

    if (!$currenttime) {
        $currenttime = time();
    }
    $currentdate = usergetdate($currenttime);
    if ($step != 1) {
        $currentdate['minutes'] = ceil($currentdate['minutes']/$step)*$step;
    }
    for ($i=0; $i<=23; $i++) {
        $hours[$i] = sprintf("%02d",$i);
    }
    for ($i=0; $i<=59; $i+=$step) {
        $minutes[$i] = sprintf("%02d",$i);
    }
    choose_from_menu($hours,   $hour,   $currentdate['hours'],   "");
    choose_from_menu($minutes, $minute, $currentdate['minutes'], "");
}

function print_timer_selector($timelimit = 0, $unit = "") {
/// Prints time limit value selector

    global $CFG;
    $minutes=array();
    if ($unit) {
        $unit = ' '.$unit;
    }
 
    // Max timelimit is sessiontimeout - 10 minutes.
    //$maxvalue = ($CFG->sessiontimeout / 60) - 10;
    $maxvalue = (120*60 / 60) - 10;

    for ($i=1; $i<=$maxvalue; $i++) {
        $minutes[$i] = $i.$unit;
    }
    choose_from_menu($minutes, "duree", $timelimit, "Aucune");
}

function print_late_selector($timelimit = 0, $unit = "",$limite=15) {
/// Prints time limit value selector

    global $CFG;
    $minutes=array();
    if ($unit) {
        $unit = ' '.$unit;
    }
 
    // Max timelimit is sessiontimeout - 10 minutes.
    //$maxvalue = ($CFG->sessiontimeout / 60) - 10;
    $maxvalue = $limite;

    for ($i=1; $i<=$maxvalue; $i++) {
        $minutes[$i] = $i.$unit;
    }
    choose_from_menu($minutes, "retard", $timelimit, "Aucun");
}
function popup_form ($common, $options, $formname, $selected="", $nothing="choose", $help="", $helptext="", $return=false, $targetwindow="self") {
///  Implements a complete little popup form
///  $common   = the URL up to the point of the variable that changes
///  $options  = A list of value-label pairs for the popup list
///  $formname = name must be unique on the page
///  $selected = the option that is already selected
///  $nothing  = The label for the "no choice" option
///  $help     = The name of a help page if help is required
///  $helptext  = The name of the label for the help button
///  $return   = Boolean indicating whether the function should return the text
///    as a string or echo it directly to the page being rendered

// TODO:
//
//  * Make sure it's W3C conformant (<form name=""> has to go for example)
//  * Code it in a way that doesn't require JS to be on. Example code:
//        $selector .= '<form method="get" action="" style="display: inline;"><span>';
//        $selector .= '<input type="hidden" name="var" value="value" />';
//        if(!empty($morevars)) {
//            $getarray = explode('&amp;', $morevars);
//            foreach($getarray as $thisvar) {
//                $selector .= '<input type="hidden" name="'.strtok($thisvar, '=').'" value="'.strtok('=').'" />';
//            }
//        }
//        $selector .= '<select name="" onchange="form.submit();">';
//        foreach($options as $id => $text) {
//            $selector .= "\n<option value='$id'";
//            if($option->id == $selected) {
//                $selector .= ' selected';
//            }
//            $selector .= '>'.$text."</option>\n";
//        }
//        $selector .= '</select>';
//        $selector .= '<noscript id="unique_id" style="display: inline;"> <input type="submit" value="'.get_string('somestring').'" /></noscript>';
//        $selector .= '<script type="text/javascript">'."\n<!--\n".'document.getElementById("unique_id").style.display = "none";'."\n<!--\n".'</script>';
//        $selector .= '</span></form>';
//

    global $CFG;

    if (empty($options)) {
        return '';
    }

    if ($nothing == "choose") {
        $nothing = get_string("choose")."...";
    }

    $startoutput = "<form method=\"get\" target=\"_top\" name=\"$formname\">";
    $output = "<select name=\"popup\" onchange=\"$targetwindow.location=document.$formname.popup.options[document.$formname.popup.selectedIndex].value\">\n";

    if ($nothing != "") {
        $output .= "   <option value=\"javascript:void(0)\">$nothing</option>\n";
    }

    foreach ($options as $value => $label) {
        if (substr($label,0,2) == "--") {
            $output .= "   <optgroup label=\"$label\"></optgroup>";   // Plain labels
            continue;
        } else {
            $output .= "   <option value=\"$common$value\"";
            if ($value == $selected) {
                $output .= " selected=\"true\"";
            }
        }
        if ($label) {
            $output .= ">$label</option>\n";
        } else {
            $output .= ">$value</option>\n";
        }
    }
    $output .= "</select>";
    $output .= "</form>\n";

    if ($help) {
        $button = helpbutton($help, $helptext, 'moodle', true, false, '', true);
    } else {
        $button = '';
    }

    if ($return) {
        return $startoutput.$button.$output;
    } else {
        echo $startoutput.$button.$output;
    }
}



function formerr($error) {
/// Prints some red text
    if (!empty($error)) {
        echo "<font color=\"#ff0000\">$error</font>";
    }
}


?>
